#include "pinyin.h"

uint16_t getPinyinCodeIndex(uint16_t c) {
  uint16_t offset = c - PINYIN_MIN_VALUE;
  if (offset >= 0 && offset <= 20901) {
    return PINYIN_CODE[offset];
  }
  return 0;
}

uint16_t toUnicode(const std::string &hanzi) {
  uint16_t code = (hanzi[0] & 0x1F) << 12;
  code |= (hanzi[1] & 0x3F) << 6;
  code |= (hanzi[2] & 0x3F);
  return code;
};

/**
 * @see
 * https://stackoverflow.com/questions/40054732/c-iterate-utf-8-string-with-mixed-length-of-characters
 */
std::string toFullChars(const std::string &text) {
  std::string ret;
  for (size_t i = 0; i < text.length();) {
    int cplen = 1;
    if ((text[i] & 0xf8) == 0xf0) {
      cplen = 4;
    } else if ((text[i] & 0xf0) == 0xe0)
      cplen = 3;
    else if ((text[i] & 0xe0) == 0xc0)
      cplen = 2;
    if ((i + cplen) > text.length())
      cplen = 1;

    const std::string cur = text.substr(i, cplen);
    if (cplen == 3) {
      uint16_t code = toUnicode(cur);
      if (code == PINYIN_CODE_12295) {
        ret += PINYIN_12295;
      } else if (code >= PINYIN_MIN_VALUE && code <= PINYIN_MAX_VALUE) {
        const std::string pinyin = PINYIN_TABLE[getPinyinCodeIndex(code)];
        if (!pinyin.empty()) {
          ret += pinyin;
        } else {
          ret += cur;
        }
      }
    } else {
      ret += cur;
    }
    i += cplen;
  }
  return ret;
}